package pl.jssoft.test;

import java.io.IOException;
import java.net.URL;
import junit.framework.TestCase;
import org.jbpm.api.Configuration;
import org.jbpm.api.ProcessEngine;
import org.jbpm.api.ProcessInstance;
import org.jbpm.api.task.Task;
import org.junit.Test;


public class SimpleTest extends TestCase
{
	@Test
	/**
	 * This test will cause javax.el.PropertyNotFoundException from EL
	 * because one of process variables will be set to null
	 */
	public void testPath1()
	{		
		String deploymentId = null;		
		ProcessEngine processEngine = Configuration.getProcessEngine(); 
		String pid = null;
		
		try
		{
			// deploy process
			URL url = getClass().getClassLoader().getResource("processes/process.jpdl.xml");
			deploymentId = processEngine.getRepositoryService().createDeployment().addResourceFromInputStream("testProcess.jpdl.xml", url.openStream()).deploy();			
					
			// start process
			ProcessInstance pi = processEngine.getExecutionService().startProcessInstanceByKey("testProcess");
			pid = pi.getId();
			
			// check user task
			Task t = processEngine.getTaskService().findPersonalTasks("username").get(0);
			
			// choose simpleform path
			processEngine.getTaskService().completeTask(t.getId(), "simple");
			
			// check user task
			t = processEngine.getTaskService().findPersonalTasks("username").get(0);
						
			// in simpleform user sets only var1 and var2 is set tu null for compatibility with next java task
			processEngine.getExecutionService().setVariable(pi.getId(), "user_var1", "Some value");
			processEngine.getExecutionService().setVariable(pi.getId(), "user_var2", null);
			
			// EL exception here! 
			processEngine.getTaskService().completeTask(t.getId(), "next");							
		}
		catch (IOException e)
		{
			throw new RuntimeException(e);
		}
		catch (org.jbpm.api.JbpmException e)
		{
			// EL says "Cannot find property user_var2" 
			// Lets see if user_var2 is accessible from java
			if (processEngine.getExecutionService().getVariableNames(pid).contains("user_var2"))
			{
				System.out.println("user_var2 is seen as process var from java code");
			}
			
			String s = "asd";			
			s = (String) processEngine.getExecutionService().getVariable(pid, "user_var2");
			if (s == null) 
				System.out.println("user_var2 read '"+s+"' from java code");
			
			throw new RuntimeException(e);
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);			
		}
		finally
		{
			processEngine.getRepositoryService().deleteDeploymentCascade(deploymentId);			
		}
	}
	
	
	@Test
	/**
	 * This test will set both variables so no exception should be thrown
	 */
	public void testPath2()
	{		
		String deploymentId = null;		
		ProcessEngine processEngine = Configuration.getProcessEngine(); 
		
		try
		{
			// deploy process
			URL url = getClass().getClassLoader().getResource("processes/process.jpdl.xml");
			deploymentId = processEngine.getRepositoryService().createDeployment().addResourceFromInputStream("testProcess.jpdl.xml", url.openStream()).deploy();			
					
			// start process
			ProcessInstance pi = processEngine.getExecutionService().startProcessInstanceByKey("testProcess");
			
			// check user task
			Task t = processEngine.getTaskService().findPersonalTasks("username").get(0);
			
			// choose simpleform path
			processEngine.getTaskService().completeTask(t.getId(), "simple");
			
			// check user task
			t = processEngine.getTaskService().findPersonalTasks("username").get(0);
						
			// in simpleform user sets only var1 and var2 is set tu null for compatibility with next java task
			processEngine.getExecutionService().setVariable(pi.getId(), "user_var1", "Some value");
			processEngine.getExecutionService().setVariable(pi.getId(), "user_var2", "var2 value");
			
			// EL exception here! 
			processEngine.getTaskService().completeTask(t.getId(), "next");
			
			System.out.println("No exception!");
		}
		catch (Exception e)
		{
			throw new RuntimeException(e);			
		}
		finally
		{
			processEngine.getRepositoryService().deleteDeploymentCascade(deploymentId);			
		}
	}
}
